io.output( io.stderr )
module( ..., package.seeall )

local ut = require( 'udunits2' )
local uts = require( 'udunits2.system' )
local utu = require( 'udunits2.unit' )

setup = _G.setup


function test_same_system( )

   local system = assert( uts.new() )

   local unit1 = assert( system:new_base_unit() );
   local unit2 = assert( system:new_base_unit() );

   assert_true( unit1:same_system( unit2 ) )

   assert_true( utu.same_system( unit1, unit2 ) )


end

function test_compare( )

   local system = assert( uts.new() )

   local unit1 = assert( system:new_base_unit() )

   assert_true( unit1:map_name_to_unit( 'foo', ut.ASCII ), 'map name' )

   local unit2 = system:parse( "20 foo" )

   assert_lt( 1, unit1:compare( unit2 ), ':compare' )
   assert_lt( 1, utu.compare( unit1, unit2 ), '.compare' )

   assert_true( unit1:are_convertible( unit2 ), ':are_convertible' )
   assert_true( utu.are_convertible( unit1, unit2 ), '.are_convertible' )


end

function test_multiply( )

   local system = assert( uts.new() )

   local one = assert( system:get_dimensionless_unit_one() );
   local two = assert( system:parse( "2" ) )

   assert_lt( 1, one:compare( two:multiply( one ) ), ':multiply' )
   assert_lt( 1, one:compare( utu.multiply( two, one ) ), '.multiply' )

end

function test_divide( )

   local system = assert( uts.new() )

   local one = assert( system:get_dimensionless_unit_one() );
   local two = assert( system:parse( "2" ) )
   local four = assert( system:parse( "4" ) )

   assert_equal( 0, two:compare( four:divide( two ) ), ':divide' )
   assert_equal( 0, two:compare( utu.divide(four, two ) ), '.divide' )

end

function test_scale()

   local system = assert( uts.new() )

   local meter = assert( system:new_base_unit() );
   local kilometer = meter:scale( 1000 );

   assert_lt( 1, meter:compare( kilometer ) )

   assert( meter:map_name_to_unit( 'm', ut.ASCII ) )

   local unit = assert( system:parse( '1000 m' ) )

   assert_equal( 0, unit:compare( kilometer ) );


end

function test_offset()

   local system = assert( uts.new() )

   local base = assert( system:new_base_unit() );
   local offset = base:offset( -33 );

   assert( base:map_name_to_unit( 'base', ut.ASCII ) )

   local unit = assert( system:parse( '33 base' ) )

   local converter = base:get_converter( offset )

   assert_true( ut.isa( converter, 'converter' ), 'is a converter?' )

   local value = converter:convert( 0 )

   assert_equal( 1, unit:compare( offset ) );


end

